@@ -4,7 +4,6 @@ from __future__ import annotations
 import collections
 import collections.abc
 
-
 import dataclasses
 from dataclasses import dataclass, make_dataclass, is_dataclass
 
@@ -254,21 +253,24 @@ class TypesConverterCursorFactory:
                     bindings: apsw.Bindings | None = None,
                     *,
                     can_cache: bool = True,
-                    prepare_flags: int = 0) -> apsw.Cursor:
+                    prepare_flags: int = 0,
+                    explain: int = -1) -> apsw.Cursor:
             """Executes the statements doing conversions on supplied and returned values
 
             See :meth:`apsw.Cursor.execute` for parameter details"""
             return super().execute(statements,
                                    self.factory.wrap_bindings(bindings),
                                    can_cache=can_cache,
-                                   prepare_flags=prepare_flags)
+                                   prepare_flags=prepare_flags,
+                                   explain=explain)
 
         def executemany(self,
                         statements: str,
                         sequenceofbindings: Sequence[apsw.Bindings],
                         *,
                         can_cache: bool = True,
-                        prepare_flags: int = 0) -> apsw.Cursor:
+                        prepare_flags: int = 0,
+                        explain: int = -1) -> apsw.Cursor:
             """Executes the statements against each item in sequenceofbindings, doing conversions on supplied and returned values
 
             See :meth:`apsw.Cursor.executemany` for parameter details"""
@@ -276,21 +278,25 @@ class TypesConverterCursorFactory:
                 statements,
                 self.factory.wrap_sequence_bindings(sequenceofbindings),  # type: ignore[arg-type]
                 can_cache=can_cache,
-                prepare_flags=prepare_flags)
+                prepare_flags=prepare_flags,
+                explain=explain)
 
 
 def log_sqlite(*, level: int = logging.ERROR) -> None:
     """Send SQLite log messages to :mod:`logging`
 
-    :param level: level to log at (default *logging.ERROR*)
+    :param level: level to log at
 
-    This must be called before doing any operations with SQLite, otherwise a :exc:`apsw.MisuseError`
-    will be raised.  (This is a SQLite limitation, not APSW.)
+    SQLite's `logging <https://www.sqlite.org/errlog.html>`__ has many
+    useful messages, including those that aren't raised as exceptions.
     """
 
     def handler(errcode: int, message: str) -> None:
-        err_str = apsw.mapping_result_codes[errcode & 255]
+        nonlocal level
+        err_str = apsw.mapping_result_codes.get(errcode & 255, str(errcode))
         extra = {"sqlite_code": errcode, "sqlite_code_name": err_str, "sqlite_message": message}
+        if errcode & 0xff == apsw.SQLITE_WARNING:
+            level = min(level, logging.WARNING)
         logging.log(level,
                     "SQLITE_LOG: %s (%d) %s %s",
                     message,
@@ -313,6 +319,13 @@ def print_augmented_traceback(exc_type: type[BaseException],
     :param exc_value: The exception value
     :param exc_traceback: Traceback for the exception
     :param file: (default ``sys.stderr``) Where the print goes
+
+    .. code::
+
+        try:
+            ....
+        except Exception as exc:
+            apsw.ext.print_augmented_traceback(*sys.exc_info())
     """
 
     file = file or sys.stderr
@@ -452,6 +465,109 @@ def index_info_to_dict(o: apsw.IndexInfo,
     return res
 
 
+def dbinfo(db: apsw.Connection,
+           schema: str = "main") -> tuple[DatabaseFileInfo | None, JournalFileInfo | WALFileInfo | None]:
+    """Extracts fields from the database files
+
+    Based on the `file format description <https://www.sqlite.org/fileformat2.html>`__.  The
+    headers are read using :meth:`apsw.Connection.read` so you see inside encrypted, compressed,
+    zip etc formats. not necessarily the actual on disk file.
+
+    Memory databases return `None` for both.
+    """
+
+    dbinfo: DatabaseFileInfo | None = None
+    journalinfo: JournalFileInfo | WALFileInfo | None = None
+
+    try:
+        ok, header_page = db.read(schema, 0, 0, 128)
+    except apsw.SQLError:
+        ok = False
+
+    be_int = functools.partial(int.from_bytes, byteorder="big", signed=False)
+
+    be_sint = functools.partial(int.from_bytes, byteorder="big", signed=True)
+
+    def be_bool(b: bytes) -> bool:
+        return bool(be_int(b))
+
+    def be_page_size(b: bytes) -> int:
+        v = be_int(b)
+        if v == 1:
+            v = 65536
+        return v
+
+    def text_encoding(b: bytes) -> str:
+        v = be_int(b)
+        return {0: "(pending)", 1: "UTF-8", 2: "UTF-16le", 3: "UTF-16be"}.get(v, f"<< INVALID VALUE { v } >>")
+
+    if ok:
+        kw: dict[str, Any] = {"filename": db.filename}
+        for name, offset, size, converter in (
+            ("header", 0, 16, bytes),
+            ("page_size", 16, 2, be_page_size),
+            ("write_format", 18, 1, be_int),
+            ("read_format", 19, 1, be_int),
+            ("reserved_bytes", 20, 1, be_int),
+            ("file_change_counter", 24, 4, be_int),
+            ("page_count", 28, 4, be_int),
+            ("freelist_pages", 36, 4, be_int),
+            ("schema_cookie", 40, 4, be_int),
+            ("schema_format", 44, 4, be_int),
+            ("default_cache_size", 48, 4, be_int),
+            ("autovacuum_top_root", 52, 4, be_int),
+            ("text_encoding", 56, 4, text_encoding),
+            ("user_version", 60, 4, be_int),
+            ("incremental_vacuum", 64, 4, be_bool),
+            ("application_id", 68, 4, be_int),
+            ("version_valid_for", 92, 4, be_int),
+            ("sqlite_version", 96, 4, be_int),
+        ):
+            b = header_page[offset:offset + size]
+            kw[name] = converter(b)  # type: ignore [operator]
+        dbinfo = DatabaseFileInfo(**kw)
+
+    try:
+        ok, journal_page = db.read(schema, 1, 0, 32)
+    except apsw.SQLError:
+        ok = False
+
+    if ok:
+        kw: dict[str, Any] = {}  # type: ignore [no-redef]
+        if db.pragma("journal_mode") == "wal":
+            kw["filename"] = db.filename_wal
+            for name, offset, size, converter in (
+                ("magic_number", 0, 4, be_int),
+                ("format_version", 4, 4, be_int),
+                ("page_size", 8, 4, be_page_size),
+                ("checkpoint_sequence_number", 12, 4, be_int),
+                ("salt_1", 16, 4, be_int),
+                ("salt_2", 20, 4, be_int),
+                ("checksum_1", 24, 4, be_int),
+                ("checksum_2", 28, 4, be_int),
+            ):
+                b = journal_page[offset:offset + size]
+                kw[name] = converter(b)  # type: ignore [operator]
+            journalinfo = WALFileInfo(**kw)
+        else:
+            header_valid = lambda b: b == b"\xd9\xd5\x05\xf9\x20\xa1\x63\xd7"
+            kw["filename"] = db.filename_journal
+            for name, offset, size, converter in (
+                ("header", 0, 8, bytes),
+                ("header_valid", 0, 8, header_valid),
+                ("page_count", 8, 4, be_sint),
+                ("random_nonce", 12, 4, be_int),
+                ("initial_pages", 16, 4, be_int),
+                ("sector_size", 20, 4, be_int),
+                ("page_size", 24, 4, be_int),
+            ):
+                b = journal_page[offset:offset + size]
+                kw[name] = converter(b)  # type: ignore [operator]
+            journalinfo = JournalFileInfo(**kw)
+
+    return dbinfo, journalinfo
+
+
 def format_query_table(db: apsw.Connection,
                        query: str,
                        bindings: apsw.Bindings | None = None,
@@ -1039,6 +1155,13 @@ def make_virtual_module(db: apsw.Connection,
                 self.repr_invalid = module.repr_invalid
                 self.num_columns = len(self.columns)
                 self.access = self.module.column_access
+                col_func = f"_Column_{ self.access.name }"
+                f = getattr(self, col_func, self.Column)
+                if self.repr_invalid:
+                    setattr(self, "Column", self._Column_repr_invalid)
+                    setattr(self, "_Column_get", f)
+                else:
+                    setattr(self, "Column", f)
 
             def Filter(self, idx_num: int, idx_str: str, args: tuple[apsw.SQLiteValue]) -> None:
                 params: dict[str, apsw.SQLiteValue] = self.param_values.copy()
@@ -1061,22 +1184,37 @@ def make_virtual_module(db: apsw.Connection,
                     self.iterating = None
 
             def Column(self, which: int) -> apsw.SQLiteValue:
+                # This is the specification/documentation for the custom
+                # versions which should produce exactly the same output
                 if which >= self.num_columns:
                     return self.hidden_values[which - self.num_columns]
-                v: Any = None
-                if self.access == VTColumnAccess.By_Index:
+                if self.access is VTColumnAccess.By_Index:
                     v = self.current_row[which]
-                elif self.access == VTColumnAccess.By_Name:
+                elif self.access is VTColumnAccess.By_Name:
                     v = self.current_row[self.columns[which]]
-                elif self.access == VTColumnAccess.By_Attr:
+                elif self.access is VTColumnAccess.By_Attr:
                     v = getattr(self.current_row, self.columns[which])
-                if self.repr_invalid:
-                    try:
-                        apsw.format_sql_value(v)
-                    except TypeError:
-                        v = repr(v)
+                if self.repr_invalid and v is not None and not isinstance(v, (int, float, str, bytes)):
+                    v = repr(v)
                 return v  # type: ignore[no-any-return]
 
+            def _Column_repr_invalid(self, which: int) -> apsw.SQLiteValue:
+                v = self._Column_get(which) # type: ignore[attr-defined]
+                return v if v is None or isinstance(v, (int, float, str, bytes)) else repr(v)
+
+            def _Column_By_Attr(self, which: int) -> apsw.SQLiteValue:
+                return getattr(
+                    self.current_row,
+                    self.columns[which]) if which < self.num_columns else self.hidden_values[which - self.num_columns]
+
+            def _Column_By_Name(self, which: int) -> apsw.SQLiteValue:
+                return self.current_row[self.columns[which]] if which < self.num_columns else self.hidden_values[
+                    which - self.num_columns]
+
+            def _Column_By_Index(self, which: int) -> apsw.SQLiteValue:
+                return self.current_row[which] if which < self.num_columns else self.hidden_values[which -
+                                                                                                   self.num_columns]
+
             def Next(self) -> None:
                 try:
                     self.current_row = next(self.iterating)  # type: ignore[arg-type]
@@ -1312,7 +1450,7 @@ def query_info(db: apsw.Connection,
 
     if explain and not res["is_explain"]:
         vdbe: list[VDBEInstruction] = []
-        for row in cur.execute("EXPLAIN " + res["first_query"], bindings):
+        for row in cur.execute(res["first_query"], bindings, explain=1):
             vdbe.append(
                 VDBEInstruction(**dict((v[0][0], v[1]) for v in zip(cur.getdescription(), row) if v[1] is not None)))
         res["explain"] = vdbe
@@ -1321,7 +1459,7 @@ def query_info(db: apsw.Connection,
         subn = "sub"
         byid: Any = {0: {"detail": "QUERY PLAN"}}
 
-        for row in cur.execute("EXPLAIN QUERY PLAN " + res["first_query"], bindings):
+        for row in cur.execute(res["first_query"], bindings, explain=2):
             node = dict((v[0][0], v[1]) for v in zip(cur.getdescription(), row) if v[0][0] != "notused")
             assert len(node) == 3  # catch changes in returned format
             parent: list[str | dict[str, Any]] = byid[node["parent"]]
@@ -1432,3 +1570,96 @@ class VDBEInstruction:
     "Fourth opcode parameter"
     p5: int | None = None
     "Fifth opcode parameter"
+
+
+@dataclass
+class DatabaseFileInfo:
+    """Information about the main database file returned by :meth:`dbinfo`
+
+    See `file format description <https://www.sqlite.org/fileformat.html#the_database_header>`__"""
+    "database filena name"
+    filename: str
+    "Header string"
+    header: bytes
+    "The database page size in bytes"
+    page_size: int
+    "File format write version. 1 for legacy; 2 for WAL"
+    write_format: int
+    "File format read version. 1 for legacy; 2 for WAL."
+    read_format: int
+    'Bytes of unused "reserved" space at the end of each page. Usually 0'
+    reserved_bytes: int
+    "File change counter"
+    file_change_counter: int
+    "Size of the database file in pages"
+    page_count: int
+    "Total number of freelist pages"
+    freelist_pages: int
+    "The schema cookie"
+    schema_cookie: int
+    "The schema format number. Supported schema formats are 1, 2, 3, and 4"
+    schema_format: int
+    "Default page cache size"
+    default_cache_size: int
+    "The page number of the largest root b-tree page when in auto-vacuum or incremental-vacuum modes, or zero otherwise"
+    autovacuum_top_root: int
+    "The database text encoding"
+    text_encoding: str
+    'The "user version" as read and set by the user_version pragma.'
+    user_version: int
+    "True (non-zero) for incremental-vacuum mode. False (zero) otherwise."
+    incremental_vacuum: bool
+    'The "Application ID" set by PRAGMA application_id'
+    application_id: int
+    "The version-valid-for number."
+    version_valid_for: int
+    "SQLite version that lost wrote"
+    sqlite_version: int
+
+
+@dataclass
+class JournalFileInfo:
+    """Information about the rollback journal returned by :meth:`dbinfo`
+
+    See the `file format description <https://www.sqlite.org/fileformat2.html#the_rollback_journal>`__"""
+    "journal file name"
+    filename: str
+    "Header string"
+    header: bytes
+    'The "Page Count" - The number of pages in the next segment of the journal, or -1 to mean all content to the end of the file'
+    header_valid: bool
+    "If the header is the expected bytes"
+    page_count: int
+    "A random nonce for the checksum"
+    random_nonce: int
+    "Initial size of the database in pages"
+    initial_pages: int
+    "Size of a disk sector assumed by the process that wrote this journal"
+    sector_size: int
+    "Size of pages in this journal"
+    page_size: int
+
+
+@dataclass
+class WALFileInfo:
+    """Information about the rollback journal returned by :meth:`dbinfo`
+
+    See the `file format description <https://www.sqlite.org/fileformat2.html#wal_file_format>`__"""
+    "WAL file name"
+    filename: str
+    "Magic number"
+    magic_number: int
+    "File format version. Currently 3007000"
+    format_version: int
+    "Database page size"
+    page_size: int
+    "Checkpoint sequence number"
+    checkpoint_sequence_number: int
+    "Salt-1: random integer incremented with each checkpoint"
+    salt_1: int
+    "Salt-2: a different random number for each checkpoint"
+    salt_2: int
+    "Checksum-1: First part of a checksum on the first 24 bytes of header"
+    checksum_1: int
+    "Checksum-2: Second part of the checksum on the first 24 bytes of header"
+    checksum_2: int
